<?php

/**
 * Mail class
 *
 * @author McArrow
 */
class ivMail
{
	/**
	 * Email body
	 * @access private
	 * @var string
	 */
	var $_body = '';

	/**
	 * Array of recipients
	 * @access private
	 * @var array
	 */
	var $_to = array();
	
	/**
	 * Sender name and email 
	 * @access private
	 * @var array
	 */
	var $_from = null;

	/**
	 * Email subject
	 * @access private
	 * @var string
	 */
	var $_subject = '';

	var $_lastError = false;

	/**
	 * Returns email body
	 *
	 * @return string
	 */
	function getBody()
	{
		return $this->_body;
	}

	/**
	 * Sets email body
	 *
	 * @param string $body
	 */
	function setBody($body)
	{
		$this->_body = $body;
	}

	/**
	 * Returns recipients
	 *
	 * @return array
	 */
	function getTo()
	{
		return $this->_to;
	}

	/**
	 * Adds recipient
	 *
	 * @param string $email
	 * @param string $name
	 */
	function addTo($email, $name = '')
	{
		$this->_to[$email] = $name;
	}

	/**
	 * Clears recipients
	 *
	 */
	function clearTo()
	{
		$this->_to = array();
	}

	/**
	 * Returns sender
	 *
	 * @return array|null
	 */
	function getFrom()
	{
		return $this->_from;
	}
	
	/**
	 * Sets sender
	 *
	 * @param string $email
	 * @param string $name
	 */
	function setFrom($email, $name = '')
	{
		if (is_null($this->_from)) {
			$this->_from = array('name' => $name, 'email' => $email);
		} else {
			trigger_error('From 字段已设置了两次', E_USER_ERROR);
		}
	}
	
	/**
	 * Returns email subject
	 *
	 * @return string
	 */
	function getSubject()
	{
		return $this->_subject;
	}

	/**
	 * Sets email subject
	 *
	 * @param string $subject
	 */
	function setSubject($subject)
	{
		$this->_subject = $subject;
	}

	/**
	 * Sends email
	 *
	 * @return boolean Operation status
	 */
	function send()
	{
		$from = $this->getFrom();
		if (is_null($from)) {
			trigger_error('From 字段还没有设置', E_USER_ERROR);
		}

		$phpMailer = new PHPMailer();
		
		$smtp = ivMail::getSmtpConfig();
		
		if ($smtp['enabled']['value']) {
			$phpMailer->IsSMTP();
			$phpMailer->Host = $smtp['host']['value'];
			$phpMailer->Port = $smtp['port']['value'];
			$phpMailer->Helo = $smtp['helo']['value'];
			$phpMailer->SMTPSecure = $smtp['secure']['value'];
			$phpMailer->SMTPAuth = $smtp['auth']['value'];
			$phpMailer->Username = $smtp['username']['value'];
			$phpMailer->Password = $smtp['password']['value'];
			$phpMailer->Timeout = $smtp['timeout']['value'];
		}

		$phpMailer->IsHTML(true);
		$phpMailer->From = $from['email'];
		$phpMailer->FromName = $from['name'];
		$phpMailer->AddReplyTo($from['email'], $from['name']);
		$phpMailer->CharSet = 'UTF-8';
		$phpMailer->Subject = $this->getSubject();
		$phpMailer->Sender = $from['email'];
		$phpMailer->Body = $this->getBody();

		$result = true;
		foreach ($this->getTo() as $toEmail => $toName) {
			$phpMailer->ClearAddresses();
			$phpMailer->AddAddress($toEmail, $toName);
			ob_start();
			$result &= $phpMailer->Send();
			ob_clean();
			if ($phpMailer->IsError()) {
				$this->_lastError = $phpMailer->ErrorInfo;
			}
		}

		return $result;
	}
	
	function getLastError()
	{
		return $this->_lastError;
	}

	/**
	 * Returns SMTP config
	 *
	 * @static
	 *
	 * @return array
	 */
	function getSmtpConfig()
	{
		$config = array(
			'enabled' => array(
				'value' => false,
				'type' => 'boolean',
				'description' => '开启SMTP功能',
			),
			'host' => array(
				'value' => 'smtp.yourdomain.com',
				'type' => 'string',
				'description' => 'SMTP主机。所有主机必须由分号分隔。您也可以指定使用这种格式：[主机名：端口]（如“smtp1.example.com：25; smtp2.example.com”）为每个主机的不同端口。',
			),
			'username' => array(
				'value' => 'yourname@yourdomain',
				'type' => 'string',
				'description' => 'SMTP 用户名',
			),
			'password' => array(
				'value' =>'yourpassword',
				'type' => 'string',
				'description' => 'SMTP 密码',
			),
			'port' => array(
				'value' => 25,
				'type' => 'integer',
				'description' => '默认的SMTP服务器端口，通常是25',
			),
			'secure' => array(
				'value' => '',
				'type' => 'string',
				'description' => '连接前缀。选项是“”，“SSL的”或“tls的”',
			),
			'auth' => array(
				'value' => true,
				'type' => 'boolean',
				'description' => 'SMTP验证。使用用户名和密码值',
			),
			'timeout' => array(
				'value' => 10,
				'type' => 'integer',
				'description' => 'SMTP服务器超时值，此功能不适用Win32版本',
			),
			'helo' => array(
				'value' => '',
				'type' => 'string',
				'description' => 'SMTP服务器的HELO域 (主机使用默认值)',
			),
		);

		if (file_exists(USER_DIR . 'smtp.php')) {
			include(USER_DIR . 'smtp.php');
			
			if (isset($smtp)) {
				foreach ($smtp as $name => $value) {
					if (array_key_exists($name, $config)) {
						$config[$name]['value'] = $value;
					}
				}
			}
		}

		return $config;
	}

	/**
	 * Saves SMTP config
	 *
	 * @static
	 *
	 * @param  array   $smtp
	 * @return boolean Operation status
	 */
	function saveSmtpConfig($smtp)
	{
		$config = ivMail::getSmtpConfig();
		
		foreach ($smtp as $name => $value) {
			if (array_key_exists($name, $config)) {
				$config[$name]['value'] = $value;
			}
		}
		
		$fileContent = "<?php\n\n";
		foreach ($config as $name => $data) {
			$fileContent .= "/**
 * {$data['description']}
 */
\$smtp['$name'] = " . ivMail::_configValueToString($data) . ";

";
		}
		
		$result = iv_file_put_contents(USER_DIR . 'smtp.php', $fileContent);
		return ($result !== false);
	}

	/**
	 * Returns string representation of smtp config value
	 *
	 * @static
	 * @private
	 *
	 * @param  array  $data
	 * @return string
	 */
	function _configValueToString($data)
	{
		switch ($data['type']) {
			case 'boolean':
				return $data['value'] ? 'true' : 'false';
				break;
			case 'integer':
				return $data['value'];
				break;
			default:
				return "'{$data['value']}'";
				break;
		}
	}

}
?>